==> CRTL-0.13.4-py3-none-any.whl: crtl/components/discrepancy_rectifier.py <==
from collections import defaultdict
import json
import pathlib
from typing import Callable, DefaultDict, Dict, List, Optional, Union
from crtl.lkb.lkb import Concept, LexicalKnowledgeBase
from spacy.tokens import Doc, Span, SpanGroup
from spacy.language import Language
from spacy.util import registry
from spacy.pipeline import Pipe


class DiscrepancyRectifier(Pipe):
    def initialize(
        self,
        get_examples=None,
        nlp=None,
        lkb_getter: Optional[Callable[[], LexicalKnowledgeBase]] = None,
        high_level_concept_ids: Optional[Union[List[str], List[int]]] = None,
    ):
        assert high_level_concept_ids
        assert lkb_getter
        lkb = lkb_getter()
        self.kb_id_to_name: Dict[Union[str, int], Union[str, int]] = {}
        for concept_id in high_level_concept_ids:
            concept = lkb.cui_to_concept[concept_id]
            self.kb_id_to_name[concept.concept_id] = concept.canonical_name
        self.low_to_high_map = self._generate_low_to_high_mapping(
            lkb, self.kb_id_to_name
        )

    def __call__(self, doc: Doc) -> Doc:
        rectified_spans = []
        for span in doc.ents:
            if span.kb_id_ in self.low_to_high_map:
                if not span.label_ in [
                    self.kb_id_to_name[kb_id] for kb_id in self.low_to_high_map[span.kb_id_]
                ]:
                    span.kb_id_ = "NIL"
            rectified_spans.append(span)
        doc.set_ents(rectified_spans)
        return doc

    def to_disk(self, path,exclude=tuple()):
        if not path.exists():
            path.mkdir()
        with open(path / "high_level_kb_id_to_name.json", "w+") as f:
            json.dump(self.kb_id_to_name, f)
        with open(path / "low_to_high_map.json", "w+") as f:
            json.dump(self.low_to_high_map, f)

    def from_disk(self, path,exclude=tuple()):
        with open(path / "high_level_kb_id_to_name.json") as f:
            self.kb_id_to_name = json.load(f)
        with open(path / "low_to_high_map.json") as f:
            self.low_to_high_map = json.load(f)

    def _generate_low_to_high_mapping(
        self,
        lkb: LexicalKnowledgeBase,
        kb_id_to_name: Dict[Union[str, int], Union[str, int]],
    ) -> DefaultDict[Union[str, int], List[Union[str, int]]]:
        # Function to collect the nodes in a breadth-first traversal
        low_to_high_map = defaultdict(lambda: [])
        for high_level_concept_id in kb_id_to_name.keys():
            high_level_concept = lkb.cui_to_concept[high_level_concept_id]
            low_to_high_map[high_level_concept.concept_id].append(
                high_level_concept.concept_id
            )
            descendants = self._get_descendants(high_level_concept, lkb)
            for descendant in descendants:
                low_to_high_map[descendant.concept_id].append(
                    high_level_concept.concept_id
                )
        return low_to_high_map

    @classmethod
    def _get_descendants(
        cls, concept: Concept, lkb: LexicalKnowledgeBase
    ) -> List[Concept]:
        descendants = []
        visited = set()
        queue = []
        queue.append(concept)
        visited.add(concept.concept_id)
        while queue:
            concept = queue.pop(0)
            for concept_relation in concept.related_concepts:
                child_concept = lkb.cui_to_concept[concept_relation.concept_id]
                if child_concept.concept_id not in visited:
                    if concept_relation.relation in ["inverse_isa", "has_tradename"]:
                        queue.append(child_concept)
                        descendants.append(child_concept)
                    visited.add(concept.concept_id)
        return descendants


@Language.factory(name="trend_discrepancy_rectifier", default_config={})
def make_trend_discrepancy_rectifier(nlp: Language, name: str):
    return DiscrepancyRectifier()


@registry.readers("crtl.read_concept_ids_from_json")
def make_high_level_concept_id_reader(
    path: Union[str, pathlib.Path]
) -> Union[List[str], List[int]]:
    with open(path) as f:
        return json.load(f)

==> CRTL-0.16.3-py3-none-any.whl: crtl/components/discrepancy_rectifier.py <==
from collections import defaultdict
import json
import pathlib
from typing import Callable, DefaultDict, Dict, List, Optional, Union
from crtl.lkb.lkb import Concept, LexicalKnowledgeBase
from spacy.tokens import Doc, Span, SpanGroup
from spacy.language import Language
from spacy.util import registry
from spacy.pipeline import Pipe


class DiscrepancyRectifier(Pipe):
    def initialize(
        self,
        get_examples=None,
        nlp=None,
        lkb_getter: Optional[Callable[[], LexicalKnowledgeBase]] = None,
        high_level_concept_ids: Optional[Union[List[str], List[int]]] = None,
    ):
        assert high_level_concept_ids
        assert lkb_getter
        lkb = lkb_getter()
        self.kb_id_to_name: Dict[Union[str, int], Union[str, int]] = {}
        for concept_id in high_level_concept_ids:
            concept = lkb.cui_to_concept[concept_id]
            self.kb_id_to_name[concept.concept_id] = concept.canonical_name
        self.low_to_high_map = self._generate_low_to_high_mapping(
            lkb, self.kb_id_to_name
        )

    def __call__(self, doc: Doc) -> Doc:
        rectified_spans = []
        for span in doc.ents:
            if span.kb_id_ in self.low_to_high_map:
                if not span.label_ in [
                    self.kb_id_to_name[kb_id]
                    for kb_id in self.low_to_high_map[span.kb_id_]
                ]:
                    span.kb_id_ = "NIL"
            rectified_spans.append(span)
        doc.set_ents(rectified_spans)
        return doc

    def to_disk(self, path, exclude=tuple()):
        if not path.exists():
            path.mkdir()
        with open(path / "high_level_kb_id_to_name.json", "w+") as f:
            json.dump(self.kb_id_to_name, f)
        with open(path / "low_to_high_map.json", "w+") as f:
            json.dump(self.low_to_high_map, f)

    def from_disk(self, path, exclude=tuple()):
        with open(path / "high_level_kb_id_to_name.json") as f:
            self.kb_id_to_name = json.load(f)
        with open(path / "low_to_high_map.json") as f:
            self.low_to_high_map = json.load(f)

    def _generate_low_to_high_mapping(
        self,
        lkb: LexicalKnowledgeBase,
        kb_id_to_name: Dict[Union[str, int], Union[str, int]],
    ) -> DefaultDict[Union[str, int], List[Union[str, int]]]:
        # Function to collect the nodes in a breadth-first traversal
        low_to_high_map = defaultdict(lambda: [])
        for high_level_concept_id in kb_id_to_name.keys():
            high_level_concept = lkb.cui_to_concept[high_level_concept_id]
            low_to_high_map[high_level_concept.concept_id].append(
                high_level_concept.concept_id
            )
            descendants = self._get_descendants(high_level_concept, lkb)
            for descendant in descendants:
                low_to_high_map[descendant.concept_id].append(
                    high_level_concept.concept_id
                )
        return low_to_high_map

    @classmethod
    def _get_descendants(
        cls, concept: Concept, lkb: LexicalKnowledgeBase
    ) -> List[Concept]:
        descendants = []
        visited = set()
        queue = []
        queue.append(concept)
        visited.add(concept.concept_id)
        while queue:
            concept = queue.pop(0)
            for concept_relation in concept.related_concepts:
                child_concept = lkb.cui_to_concept[concept_relation.concept_id]
                if child_concept.concept_id not in visited:
                    if concept_relation.relation in ["inverse_isa", "has_tradename"]:
                        queue.append(child_concept)
                        descendants.append(child_concept)
                    visited.add(concept.concept_id)
        return descendants


@Language.factory(name="trend_discrepancy_rectifier", default_config={})
def make_trend_discrepancy_rectifier(nlp: Language, name: str):
    return DiscrepancyRectifier()


@registry.readers("crtl.read_concept_ids_from_json")
def make_high_level_concept_id_reader(
    path: Union[str, pathlib.Path]
) -> Union[List[str], List[int]]:
    with open(path) as f:
        return json.load(f)
